@@ -7,6 +7,10 @@ class RasterioError(Exception):
     """Root exception class"""
 
 
+class InvalidArrayError(RasterioError):
+    """Raised when methods are passed invalid arrays"""
+
+
 class WindowError(RasterioError):
     """Raised when errors occur during window operations"""
 
@@ -34,7 +38,7 @@ class FileOverwriteError(FileError):
 
     def __init__(self, message):
         """Raise FileOverwriteError with message as hint."""
-        super(FileOverwriteError, self).__init__('', hint=message)
+        super().__init__('', hint=message)
 
 
 class RasterioIOError(IOError):
@@ -55,6 +59,10 @@ class NotGeoreferencedWarning(UserWarning):
     """Warn that a dataset isn't georeferenced."""
 
 
+class RPCTransformWarning(UserWarning):
+    """Error raised when GDALRPCTransform fails."""
+
+
 class ShapeSkipWarning(UserWarning):
     """Warn that an invalid or empty shape in a collection has been skipped"""
 
@@ -79,17 +87,21 @@ class GDALOptionNotImplementedError(RasterioError):
     by GDAL 1.x.
     """
 
+
 class GDALVersionError(RasterioError):
-    """Raised if the runtime version of GDAL does not meet the required
-    version of GDAL."""
+    """Raised if the runtime version of GDAL does not meet the required version of GDAL."""
 
 
 class WindowEvaluationError(ValueError):
     """Raised when window evaluation fails"""
 
 
-class RasterioDeprecationWarning(UserWarning):
-    """Rasterio module deprecations"""
+class RasterioDeprecationWarning(FutureWarning):
+    """Rasterio module deprecations
+
+    Following https://www.python.org/dev/peps/pep-0565/#additional-use-case-for-futurewarning
+    we base this on FutureWarning while continuing to support Python < 3.7.
+    """
 
 
 class RasterBlockError(RasterioError):
@@ -126,3 +138,11 @@ class ResamplingAlgorithmError(RasterioError):
 
 class TransformError(RasterioError):
     """Raised when transform arguments are invalid"""
+
+
+class WarpedVRTError(RasterioError):
+    """Raised when WarpedVRT can't be initialized"""
+
+
+class DatasetIOShapeError(RasterioError):
+    """Raised when data buffer shape is a mismatch when reading and writing"""
